@@ -31,6 +31,7 @@ const DesktopIconsUtil = imports.desktopIconsUtil;
 
 const Prefs = imports.preferences;
 const Enums = imports.enums;
+const SignalManager = imports.signalManager;
 
 const ByteArray = imports.byteArray;
 const Signals = imports.signals;
@@ -38,9 +39,9 @@ const Gettext = imports.gettext.domain('ding');
 
 const _ = Gettext.gettext;
 
-var desktopIconItem = class desktopIconItem {
+var desktopIconItem = class desktopIconItem extends SignalManager.SignalManager {
     constructor(desktopManager, fileExtra) {
-        this._signalIds = [];
+        super();
         this._desktopManager = desktopManager;
         this._fileExtra = fileExtra;
         this._loadThumbnailDataCancellable = null;
@@ -72,20 +73,20 @@ var desktopIconItem = class desktopIconItem {
     }
 
     _destroy() {
+        this._destroyed = true;
         /* Regular file data */
         if (this._queryFileInfoCancellable) {
             this._queryFileInfoCancellable.cancel();
+            this._queryFileInfoCancellable = null;
         }
 
         /* Thumbnailing */
         if (this._loadThumbnailDataCancellable) {
             this._loadThumbnailDataCancellable.cancel();
+            this._loadThumbnailDataCancellable = null;
         }
         /* Disconnect signals */
-        for (let [object, signalId] of this._signalIds) {
-            object.disconnect(signalId);
-        }
-        this._signalIds = [];
+        this.disconnectAllSignals();
         this.container.destroy();
         this.container = null;
         this._eventBox = null;
@@ -97,26 +98,28 @@ var desktopIconItem = class desktopIconItem {
         this._label = null;
         this._labelContainer = null;
         this.iconRectangle = null;
+        if (this._grid) {
+            this._grid.removeItem(this);
+            this._grid = null;
+        }
+        this._desktopManager = null;
+        this._fileExtra = null;
+        this._savedCoordinates = null;
+        this._dropCoordinates = null;
     }
 
     _onDestroy() {
         if (!this._destroyed) {
             this._destroy();
-            this._destroyed = true;
         }
     }
 
-    _connectSignal(object, signal, callback) {
-        this._signalIds.push([object, object.connect(signal, callback)]);
-    }
-
     /** *********************
      * Creators *
      ***********************/
 
     _createIconActor() {
         this.container = new Gtk.Box({orientation: Gtk.Orientation.VERTICAL, halign: Gtk.Align.CENTER});
-        this._connectSignal(this.container, 'destroy', () => this._onDestroy());
         this._eventBox = new Gtk.EventBox({visible: true, halign: Gtk.Align.CENTER});
         this._shieldEventBox = new Gtk.EventBox({visible: true, halign: Gtk.Align.CENTER});
         this._labelEventBox = new Gtk.EventBox({visible: true, halign: Gtk.Align.CENTER});
@@ -166,43 +169,43 @@ var desktopIconItem = class desktopIconItem {
          * The solution is to allow them to pass in a EventBox, used both for detecting the events and the DnD, and block them
          * in a second EventBox, located outside.
          */
-        this._connectSignal(this._shieldEventBox, 'button-press-event', (actor, event) => {
+        this.connectSignal(this._shieldEventBox, 'button-press-event', (actor, event) => {
             return true;
         });
-        this._connectSignal(this._shieldLabelEventBox, 'button-press-event', (actor, event) => {
+        this.connectSignal(this._shieldLabelEventBox, 'button-press-event', (actor, event) => {
             return true;
         });
-        this._connectSignal(this._eventBox, 'button-press-event', (actor, event) => this._onPressButton(actor, event));
-        this._connectSignal(this._eventBox, 'enter-notify-event', (actor, event) => this._onEnter(this._eventBox));
-        this._connectSignal(this._eventBox, 'leave-notify-event', (actor, event) => this._onLeave(this._eventBox));
-        this._connectSignal(this._eventBox, 'button-release-event', (actor, event) => this._onReleaseButton(actor, event));
-        this._connectSignal(this._eventBox, 'drag-motion', (widget, context, x, y, time) => {
+        this.connectSignal(this._eventBox, 'button-press-event', (actor, event) => this._onPressButton(actor, event));
+        this.connectSignal(this._eventBox, 'enter-notify-event', (actor, event) => this._onEnter(this._eventBox));
+        this.connectSignal(this._eventBox, 'leave-notify-event', (actor, event) => this._onLeave(this._eventBox));
+        this.connectSignal(this._eventBox, 'button-release-event', (actor, event) => this._onReleaseButton(actor, event));
+        this.connectSignal(this._eventBox, 'drag-motion', (widget, context, x, y, time) => {
             this.highLightDropTarget(x, y);
             this._updateDragStatus(context, time);
         });
-        this._connectSignal(this._eventBox, 'drag-leave', () => {
+        this.connectSignal(this._eventBox, 'drag-leave', () => {
             this.unHighLightDropTarget();
         });
-        this._connectSignal(this._eventBox, 'size-allocate', () => this._calculateIconRectangle());
-        this._connectSignal(this._labelEventBox, 'button-press-event', (actor, event) => this._onPressButton(actor, event));
-        this._connectSignal(this._labelEventBox, 'enter-notify-event', (actor, event) => this._onEnter(this._labelEventBox));
-        this._connectSignal(this._labelEventBox, 'leave-notify-event', (actor, event) => this._onLeave(this._labelEventBox));
-        this._connectSignal(this._labelEventBox, 'button-release-event', (actor, event) => this._onReleaseButton(actor, event));
-        this._connectSignal(this._labelEventBox, 'drag-motion', (widget, context, x, y, time) => {
+        this.connectSignal(this._eventBox, 'size-allocate', () => this._calculateIconRectangle());
+        this.connectSignal(this._labelEventBox, 'button-press-event', (actor, event) => this._onPressButton(actor, event));
+        this.connectSignal(this._labelEventBox, 'enter-notify-event', (actor, event) => this._onEnter(this._labelEventBox));
+        this.connectSignal(this._labelEventBox, 'leave-notify-event', (actor, event) => this._onLeave(this._labelEventBox));
+        this.connectSignal(this._labelEventBox, 'button-release-event', (actor, event) => this._onReleaseButton(actor, event));
+        this.connectSignal(this._labelEventBox, 'drag-motion', (widget, context, x, y, time) => {
             this.highLightDropTarget(x, y);
             this._updateDragStatus(context, time);
         });
-        this._connectSignal(this._labelEventBox, 'drag-leave', () => {
+        this.connectSignal(this._labelEventBox, 'drag-leave', () => {
             this.unHighLightDropTarget();
         });
-        this._connectSignal(this._labelEventBox, 'size-allocate', () => {
+        this.connectSignal(this._labelEventBox, 'size-allocate', () => {
             this._doLabelSizeAllocated();
         });
-        this._connectSignal(this.container, 'drag-motion', (widget, context, x, y, time) => {
+        this.connectSignal(this.container, 'drag-motion', (widget, context, x, y, time) => {
             this.highLightDropTarget(x, y);
             this._updateDragStatus(context, time);
         });
-        this._connectSignal(this.container, 'drag-leave', () => {
+        this.connectSignal(this.container, 'drag-leave', () => {
             this.unHighLightDropTarget();
         });
 
@@ -480,7 +483,7 @@ var desktopIconItem = class desktopIconItem {
         }
         widget.drag_source_set_target_list(targets);
         targets = undefined; // prevent memory leaks
-        this._connectSignal(widget, 'drag-begin', (w, context) => {
+        this.connectSignal(widget, 'drag-begin', (w, context) => {
             const scale = this._icon.get_scale_factor();
             let surf = new Cairo.ImageSurface(Cairo.SurfaceType.IMAGE, this.container.get_allocated_width() * scale, this.container.get_allocated_height() * scale);
             // setDeviceScale was introduced to GJS in version 1.69.2
@@ -522,14 +525,14 @@ var desktopIconItem = class desktopIconItem {
             this._desktopManager.onDragBegin(this);
             cr.$dispose();
         });
-        this._connectSignal(widget, 'drag-data-get', (w, context, data, info, time) => {
+        this.connectSignal(widget, 'drag-data-get', (w, context, data, info, time) => {
             let dragData = this._desktopManager.fillDragDataGet(info);
             if (dragData != null) {
                 let list = ByteArray.fromString(dragData[1]);
                 data.set(dragData[0], 8, list);
             }
         });
-        this._connectSignal(widget, 'drag-end', (w, context) => {
+        this.connectSignal(widget, 'drag-end', (w, context) => {
             this._desktopManager.onDragEnd();
         });
     }
